#ifndef COMM_H
#define COMM_H


#include "mpi.h"
#include "globals.h"


BlockVectorBuffer *allocate_block_vector_buffer( int num, int blksz );
void deallocate_block_vector_buffer( BlockVectorBuffer *bvb );


#define BUF_ID_R             0
#define BUF_ID_W1            1
#define BUF_ID_W2            2
#define BUF_ID_S1            3
#define BUF_ID_S2            4


/* Allocate memory and initialize variables */
void comm_init( int capacity, int nprow, int npcol, int myrow, int mycol );

/* Free memory and clean up */
void comm_finalize( void );

/* Add a request to the datastructure, will fail if datastructure is full */
void comm_add_request( MPI_Request req, int tag, int buf_ID );

/* Poll until no active requests */
void comm_wait_all( void );

/**
 * Wait routine. 
 * This functions waits until the following two conditions are satisfied:
 * 1) The state of the BBS matches the requested state.
 * 2) No execution is active on the BBS.
 */
void comm_wait( BufferBlockState *bbs, int state );

void comm_wait_idle( BufferBlockState *bbs );

/* Test any of the requests for completion, update datastructure and call the function
   associated with the bufferTag */
void comm_poll( int flag_blocking );

/* Initialize a BufferBlockState, i.e. set cnt to 0 , flags to 0, state to -1 */
void bbs_init( BufferBlockState *bbs );


/**
 * Communication algorithm starting functions.
 */
void startCommunicateR( int j );
void startCommunicateW( int j, int i );
void startCommunicateS( int j, int k );


#endif
