#ifndef GLOBALS_H
#define GLOBALS_H

#include "mpi.h"
#include "dsbp.h"


/* Datastructure holding an MPI tag and a tag describing which buffer the request is associated with */
typedef struct {
   int tag;
   int buf_ID;
} RequestTag;


/* Callback for request completion */
typedef void (*EventHandler)( RequestTag* );

 
/* Datastructure holding requests and associated information */
typedef struct {
   /* List of requests */
   MPI_Request *mpi_reqs;
   /* List of tag values and buffer IDs */
   RequestTag *tags;
   /* List of statuses */
   MPI_Status *statuses;
   /* List of indices */
   int *indices;

   /* Number of requests allocated for the list and the number of active requests */
   int capacity;
   int size;

   /* Mesh Communicators */
   MPI_Comm comm_all;
   MPI_Comm comm_col;
   MPI_Comm comm_row;
} CommunicationData;


/**
 * Information about a single buffer block. 
 * Includes parameters for the communication algorithm active on the block.
 */
typedef struct bufferblockstate
{
   /* Function parameters */
   union {
      /* communicateR */
      struct {
         int j; 
      } R;

      /* communicateW */
      struct {
         int j;
         int i;
      } W;
   
      /* communicateS */
      struct {
         int j;
         int k;
      } S;
   } params;
   
   /* Task count (each of the communication algorithms are resumable and this is the task counter) */
   int cnt;
   
   /* Flags */
   int is_executing;

   /* State (i.e. which iteration this blocks contents stems from) */
   int state;
} BufferBlockState;


/**
 * A BlockVectorBuffer is a vector of blocks with associated BufferBlockState:s.
 */
typedef struct block_vector_buffer_s BlockVectorBuffer;
struct block_vector_buffer_s {
   double* matrix;            /* Actual blocks */
   BufferBlockState* bbs;     /* One per block */
};


/**
 * Various global data.
 */
typedef struct 
{
   DSBPMatrix *A;

   int nprow;
   int npcol;
   int myrow;
   int mycol;

   BlockVectorBuffer *R;
   BlockVectorBuffer *W[2];
   BlockVectorBuffer *S[2];

   CommunicationData cd;
} Globals;


extern Globals globals;
extern int opt_verbose;



#endif
